#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains widget with joint functionality
"""

import logging

from Qt.QtCore import Signal, QObject
from Qt.QtWidgets import QWidget

from tpDcc import dcc
from tpDcc.libs.qt.core import contexts as qt_contexts
from tpDcc.libs.qt.widgets import layouts, label, spinbox, checkbox

from tpRigToolkit.tools.rigtoolbox.core import consts
from tpRigToolkit.tools.rigtoolbox.widgets import base

LOGGER = logging.getLogger(consts.TOOL_ID)


class JointWidget(base.CommandRigToolBoxWidget, object):

    def __init__(self, client, commands_data, parent=None):

        self._model = JointWidgetModel()

        super(JointWidget, self).__init__(
            title='Joint', commands_data=commands_data,
            controller=JointWidgetController(model=self._model, client=client), parent=parent)

        self.refresh()

    @property
    def model(self):
        return self._model

    @property
    def controller(self):
        return self._controller

    def ui(self):
        super(JointWidget, self).ui()

        self._joints_to_insert_widget = QWidget()
        joints_to_insert_layout = layouts.HorizontalLayout(spacing=0, margins=(0, 0, 0, 0))
        self._joints_to_insert_widget.setLayout(joints_to_insert_layout)
        joints_to_insert_lbl = label.BaseLabel('Num. Joints: ', parent=self)
        self._joints_to_insert_spn = spinbox.BaseSpinBox(parent=self)
        self._joints_to_insert_spn.setMinimum(1)
        self._joints_to_insert_spn.setMaximum(99999999)
        joints_to_insert_layout.addWidget(joints_to_insert_lbl)
        joints_to_insert_layout.addWidget(self._joints_to_insert_spn)

        self._create_joints_on_curve_widget = QWidget()
        create_joints_on_curve_layout = layouts.HorizontalLayout(spacing=0, margins=(0, 0, 0, 0))
        self._create_joints_on_curve_widget.setLayout(create_joints_on_curve_layout)
        create_joints_on_curve_lbl = label.BaseLabel('Num. Joints: ', parent=self)
        self._create_joints_on_curve_spn = spinbox.BaseSpinBox(parent=self)
        self._create_joints_on_curve_spn.setMinimum(1)
        self._create_joints_on_curve_spn.setMaximum(99999999)
        create_joints_on_curve_layout.addWidget(create_joints_on_curve_lbl)
        create_joints_on_curve_layout.addWidget(self._create_joints_on_curve_spn)

        self._snap_joints_to_curve_widget = QWidget()
        snap_joints_to_curve_layout = layouts.HorizontalLayout(spacing=0, margins=(0, 0, 0, 0))
        self._snap_joints_to_curve_widget.setLayout(snap_joints_to_curve_layout)
        snap_joints_to_curve_lbl = label.BaseLabel('Num. Joints: ', parent=self)
        self._snap_joints_to_curve_spn = spinbox.BaseSpinBox(parent=self)
        self._snap_joints_to_curve_spn.setMinimum(0)
        self._snap_joints_to_curve_spn.setMaximum(99999999)
        snap_joints_to_curve_layout.addWidget(snap_joints_to_curve_lbl)
        snap_joints_to_curve_layout.addWidget(self._snap_joints_to_curve_spn)

        self._joints_display_size_widget = QWidget()
        joint_display_size_layout = layouts.HorizontalLayout(spacing=0, margins=(0, 0, 0, 0))
        self._joints_display_size_widget.setLayout(joint_display_size_layout)
        joint_display_size_lbl = label.BaseLabel('Joints Size: ', parent=self)
        self._joints_display_size_spn = spinbox.BaseDoubleSpinBox(parent=self)
        self._joints_display_size_spn.setSingleStep(0.5)
        self._joints_display_size_spn.setMinimum(0.1)
        self._joints_display_size_spn.setMaximum(999)
        self._joints_display_live_cbx = checkbox.BaseCheckBox('Live', parent=self)
        joint_display_size_layout.addWidget(joint_display_size_lbl)
        joint_display_size_layout.addWidget(self._joints_display_size_spn)
        joint_display_size_layout.addWidget(self._joints_display_live_cbx)

        self._joints_to_insert_widget.setVisible(False)
        self._create_joints_on_curve_widget.setVisible(False)
        self._snap_joints_to_curve_widget.setVisible(False)
        self._joints_display_size_widget.setVisible(False)

        self.main_layout.addWidget(self._joints_to_insert_widget)
        self.main_layout.addWidget(self._create_joints_on_curve_widget)
        self.main_layout.addWidget(self._snap_joints_to_curve_widget)
        self.main_layout.addWidget(self._joints_display_size_widget)

    def setup_signals(self):
        self._joints_to_insert_spn.valueChanged.connect(self._controller.change_joints_to_insert)
        self._create_joints_on_curve_spn.valueChanged.connect(self._controller.change_joints_on_curve)
        self._snap_joints_to_curve_spn.valueChanged.connect(self._controller.change_snap_joints_to_curve)
        self._joints_display_size_spn.valueChanged.connect(self._controller.change_joints_display_size)
        self._joints_display_live_cbx.toggled.connect(self._controller.change_joints_display_size_live)

        self._model.jointsToInsertChanged.connect(self._joints_to_insert_spn.setValue)
        self._model.jointsOnCurveChanged.connect(self._create_joints_on_curve_spn.setValue)
        self._model.snapJointsToCurveChanged.connect(self._snap_joints_to_curve_spn.setValue)
        self._model.jointsDisplaySizeChanged.connect(self._on_joints_display_size_changed)
        self._model.jointsDisplaySizeLiveChanged.connect(self._joints_display_live_cbx.setChecked)

    def refresh(self):
        self._joints_to_insert_spn.setValue(self._model.joints_to_insert)
        self._joints_display_size_spn.setValue(self._model.joints_display_size)
        self._joints_display_live_cbx.setChecked(self._model.joints_display_size_live)
        self._create_joints_on_curve_spn.setValue(self._model.joints_on_curve)
        self._snap_joints_to_curve_spn.setValue(self._model.snap_joints_to_curve)

    def _on_joints_display_size_changed(self, value):
        live = self._model.joints_display_size_live

        with qt_contexts.block_signals(self._joints_display_size_spn):
            self._joints_display_size_spn.setValue(value)

        if live:
            self._controller.joint_display_size()

    # def _setup_mirror_tools(self):

    #     mirror_icon = resources.icon('mirror', extension='png')
    #     mirror_all_icon = resources.icon('mirror_all', extension='png')
    #     mirror_selection_icon = resources.icon('mirror_selection', extension='png')
    #     mirror_joint_btn = buttons.BaseToolButton(parent=self)
    #     mirror_joint_btn.setPopupMode(QToolButton.MenuButtonPopup)
    #     mirror_joint_btn.setToolButtonStyle(Qt.ToolButtonTextUnderIcon)
    #     mirror_joint_btn.setIcon(mirror_icon)
    #     mirror_joint_btn.setText('Mirror Joint')
    #     mirror_joint_menu = QMenu(mirror_joint_btn)
    #     mirror_joint_btn.setMenu(mirror_joint_menu)
    #     mirror_all_joints_action = QAction(mirror_all_icon, 'Mirror All Joints', mirror_joint_menu)
    #     mirror_selected_joints_action = QAction(mirror_selection_icon, 'Mirror Selected Joints', mirror_joint_menu)
    #     mirror_hierarchy_joints_action = QAction(mirror_selection_icon, 'Mirror Hierarchy Joints', mirror_joint_menu)
    #     mirror_joint_menu.addAction(mirror_all_joints_action)
    #     mirror_joint_menu.addAction(mirror_selected_joints_action)
    #     mirror_joint_menu.addAction(mirror_hierarchy_joints_action)
    #     mirror_joint_btn.clicked.connect(self._controller.mirror_all_joints)
    #     mirror_selected_joints_action.triggered.connect(self._controller.mirror_selected_joints)
    #     mirror_all_joints_action.triggered.connect(self._controller.mirror_all_joints)
    #     mirror_hierarchy_joints_action.triggered.connect(self._controller.mirror_hierarchy_joints)
    #     self.mirror_layout.addWidget(mirror_joint_btn)
    #
    #     mirror_create_icon = resources.icon('mirror2', extension='png')
    #     mirror_create_all_icon = resources.icon('mirror2', extension='png')
    #     mirror_create_btn = buttons.BaseToolButton(parent=self)
    #     mirror_create_btn.setPopupMode(QToolButton.MenuButtonPopup)
    #     mirror_create_btn.setToolButtonStyle(Qt.ToolButtonTextUnderIcon)
    #     mirror_create_btn.setIcon(mirror_create_icon)
    #     mirror_create_btn.setText('Mirror Create')
    #     mirror_create_menu = QMenu(mirror_create_btn)
    #     mirror_create_btn.setMenu(mirror_create_menu)
    #     mirror_create_all_joints_action = QAction(
    #         mirror_create_all_icon, 'Mirror Create All Joints', mirror_create_menu)
    #     mirror_create_menu.addAction(mirror_create_all_joints_action)
    #     mirror_create_btn.clicked.connect(self._controller.mirror_create_all_joints)
    #     mirror_create_all_joints_action.triggered.connect(self._controller.mirror_create_all_joints)
    #     self.mirror_layout.addWidget(mirror_create_btn)
    #
    #     return mirror_widget
    #
    # def _setup_pose_tools(self):
    #     pose_widget = QWidget()
    #     self.pose_layout = layouts.FlowLayout()
    #     self.pose_layout.setAlignment(Qt.AlignLeft)
    #     pose_widget.setLayout(self.pose_layout)
    #
    #     go_to_bindpose_btn = buttons.BaseButton('Go to Bindpose', parent=self)
    #     self.pose_layout.addWidget(go_to_bindpose_btn)
    #
    #     return pose_widget


class JointWidgetModel(QObject, object):

    jointsToInsertChanged = Signal(int)
    jointsOnCurveChanged = Signal(int)
    snapJointsToCurveChanged = Signal(int)
    jointsDisplaySizeChanged = Signal(float)
    jointsDisplaySizeLiveChanged = Signal(bool)

    def __init__(self):
        super(JointWidgetModel, self).__init__()

        self._joints_to_insert = 1
        self._joints_on_curve = 10
        self._snap_joints_to_curve = 0
        self._joints_display_size_live = True

        try:
            self._joints_display_size = dcc.client().get_joint_display_size()
        except Exception:
            LOGGER.warning('Impossible to retrieve current DCC joint display size. Using 1.0 by default ...')
            self._joints_display_size = 1.0

    @property
    def joints_to_insert(self):
        return self._joints_to_insert

    @joints_to_insert.setter
    def joints_to_insert(self, value):
        self._joints_to_insert = int(value)
        self.jointsToInsertChanged.emit(self._joints_to_insert)

    @property
    def joints_on_curve(self):
        return self._joints_on_curve

    @joints_on_curve.setter
    def joints_on_curve(self, value):
        self._joints_on_curve = int(value)
        self.jointsOnCurveChanged.emit(self._joints_on_curve)

    @property
    def snap_joints_to_curve(self):
        return self._snap_joints_to_curve

    @snap_joints_to_curve.setter
    def snap_joints_to_curve(self, value):
        self._snap_joints_to_curve = int(value)
        self.snapJointsToCurveChanged.emit(self._snap_joints_to_curve)

    @property
    def joints_display_size_live(self):
        return self._joints_display_size_live

    @joints_display_size_live.setter
    def joints_display_size_live(self, flag):
        self._joints_display_size_live = bool(flag)
        self.jointsDisplaySizeLiveChanged.emit(self._joints_display_size_live)

    @property
    def joints_display_size(self):
        return self._joints_display_size

    @joints_display_size.setter
    def joints_display_size(self, value):
        self._joints_display_size = float(value)
        self.jointsDisplaySizeChanged.emit(self._joints_display_size)


class JointWidgetController(object):

    def __init__(self, model, client):
        super(JointWidgetController, self).__init__()

        self._model = model
        self._client = client

    @property
    def model(self):
        return self._model

    @property
    def client(self):
        return self._client

    def change_joints_to_insert(self, value):
        self._model.joints_to_insert = value

    def change_joints_on_curve(self, value):
        self._model.joints_on_curve = value

    def change_snap_joints_to_curve(self, value):
        self._model.snap_joints_to_curve = value

    def change_joints_display_size(self, value):
        self._model.joints_display_size = value

    def change_joints_display_size_live(self, flag):
        self._model.joints_display_size_live = flag

    def start_joint_tool(self):
        return self._client.start_joint_tool()

    def create_new_joint_on_center(self):
        return self._client.create_new_joint_on_center()

    def create_new_joints_on_selected_components(self):
        return self._client.create_new_joints_on_selected_components()

    def insert_joints(self):
        joint_count = self._model.joints_to_insert
        return self._client.insert_joints(num_joints=joint_count)

    def create_joints_on_curve(self):
        joints_on_curve = self._model.joints_on_curve
        return self._client.create_joints_on_curve(num_joints=joints_on_curve)

    def snap_joints_to_curve(self):
        joints_to_snap = self._model.snap_joints_to_curve
        return self._client.snap_joints_to_curve(num_joints=joints_to_snap)

    def toggle_local_rotation_axis(self):
        return self._client.toggle_local_rotation_axis()

    def toggle_all_local_rotation_axis(self):
        return self._client.toggle_all_local_rotation_axis()

    def toggle_selected_local_rotation_axis(self):
        return self._client.toggle_selected_local_rotation_axis()

    def on_all_local_rotation_axis(self):
        return self._client.toggle_all_local_rotation_axis(flag=True)

    def off_all_local_rotation_axis(self):
        return self._client.toggle_all_local_rotation_axis(flag=False)

    def on_selected_local_rotation_axis(self):
        return self._client.toggle_selected_local_rotation_axis(flag=True)

    def off_selected_local_rotation_axis(self):
        return self._client.toggle_selected_local_rotation_axis(flag=False)

    def toggle_joints_xray(self):
        return self._client.toggle_joints_xray()

    def on_joints_xray(self):
        return self._client.set_joints_xray(True)

    def off_joints_xray(self):
        return self._client.set_joints_xray(False)

    def joint_display_size(self):
        joints_display_size = self._model.joints_display_size
        return self._client.set_joints_display_size(joints_display_size)

    def select_hierarchy(self):
        return self._client.select_hierarchy()

    def orient_joints(self):
        return self._client.orient_joints()

    def orient_all_joints(self):
        return self._client.orient_all_joints()

    def orient_selected_joints(self):
        return self._client.orient_selected_joints()

    def add_orient_data(self):
        return self._client.add_orient_data()

    def add_orient_data_all_joints(self):
        return self._client.add_orient_data_all_joints()

    def add_orient_data_selected_joints(self):
        return self._client.add_orient_data_selected_joints()

    def clean_orient_data(self):
        return self._client.clean_orient_data()

    def clean_orient_data_all_joints(self):
        return self._client.clean_orient_data_all_joints()

    def clean_orient_data_selected_joints(self):
        return self._client.clean_orient_data_selected_joints()

    def zero_joint_orient(self):
        return self._client.zero_joint_orient()

    def zero_joint_orient_all_joints(self):
        return self._client.zero_joint_orient_all_joints()

    def zero_joint_orient_selected_joints(self):
        return self._client.zero_joint_orient_selected_joints()

    def orient_tool(self):
        return self._client.orient_tool()

    def mirror_joints(self):
        return self._client.mirror_joints()

    def mirror_all_joints(self):
        return self._client.mirror_all_joints()

    def mirror_selected_joints(self):
        return self._client.mirror_selected_joints()

    def mirror_hierarchy_joints(self):
        return self._client.mirror_hierarchy_joints()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains rig toolbox client implementation
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"

from tpDcc.core import client
from tpDcc.managers import tools


class RigToolboxClient(client.DccClient, object):

    PORT = 19344

    # =================================================================================================================
    # RENAMER
    # =================================================================================================================

    def setup_renamer_client(self):

        cmd = {
            'cmd': 'setup_renamer_client'
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return False

        return reply_dict['success']

    # =================================================================================================================
    # CONTROL RIG
    # =================================================================================================================

    def setup_control_rig_client(self):

        cmd = {
            'cmd': 'setup_control_rig_client'
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return False

        return reply_dict['success']

    # =================================================================================================================
    # SYMMESH
    # =================================================================================================================

    def setup_symmesh_client(self):

        cmd = {
            'cmd': 'setup_symmesh_client'
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return False

        return reply_dict['success']

    # =================================================================================================================
    # MAYA - GENERAL
    # =================================================================================================================

    def delete_history(self, transforms=None):
        cmd = {
            'cmd': 'delete_history',
            'transforms': transforms
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def freeze_transforms(self, transforms=None):
        cmd = {
            'cmd': 'freeze_transforms',
            'transforms': transforms
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def move_pivot_to_zero(self, transforms=None):
        cmd = {
            'cmd': 'move_pivot_to_zero',
            'transforms': transforms
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def lock_all_transforms(self, transforms=None):
        cmd = {
            'cmd': 'lock_all_transforms',
            'transforms': transforms
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def lock_translation(self, transforms=None):
        cmd = {
            'cmd': 'lock_translation',
            'transforms': transforms
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def lock_rotation(self, transforms=None):
        cmd = {
            'cmd': 'lock_rotation',
            'transforms': transforms
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def lock_scale(self, transforms=None):
        cmd = {
            'cmd': 'lock_scale',
            'transforms': transforms
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def lock_visibility(self, transforms=None):
        cmd = {
            'cmd': 'lock_visibility',
            'transforms': transforms
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def unlock_all_transforms(self, transforms=None):
        cmd = {
            'cmd': 'unlock_all_transforms',
            'transforms': transforms
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def unlock_translation(self, transforms=None):
        cmd = {
            'cmd': 'unlock_translation',
            'transforms': transforms
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def unlock_rotation(self, transforms=None):
        cmd = {
            'cmd': 'unlock_rotation',
            'transforms': transforms
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def unlock_scale(self, transforms=None):
        cmd = {
            'cmd': 'unlock_scale',
            'transforms': transforms
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def unlock_visibility(self, transforms=None):
        cmd = {
            'cmd': 'unlock_visibility',
            'transforms': transforms
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def clean_student_license(self, file_path=None):
        cmd = {
            'cmd': 'clean_student_license',
            'file_path': file_path
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def match_transform(self, source_transform=None, target_transform=None):
        cmd = {
            'cmd': 'match_transform',
            'source_transform': source_transform,
            'target_transform': target_transform
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def match_translation(self, source_transform=None, target_transform=None):
        cmd = {
            'cmd': 'match_translation',
            'source_transform': source_transform,
            'target_transform': target_transform
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def match_rotation(self, source_transform=None, target_transform=None):
        cmd = {
            'cmd': 'match_rotation',
            'source_transform': source_transform,
            'target_transform': target_transform
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def match_scale(self, source_transform=None, target_transform=None):
        cmd = {
            'cmd': 'match_scale',
            'source_transform': source_transform,
            'target_transform': target_transform
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def combine_meshes(self, meshes=None, new_mesh_name=None):
        cmd = {
            'cmd': 'combine_meshes',
            'meshes': meshes,
            'new_mesh_name': new_mesh_name
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def separate_meshes(self, meshes=None):
        cmd = {
            'cmd': 'separate_meshes',
            'meshes': meshes,
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def mirror_meshes(self, meshes=None):
        cmd = {
            'cmd': 'mirror_meshes',
            'meshes': meshes,
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def mirror_meshes_options(self):
        cmd = {
            'cmd': 'mirror_meshes_options'
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def open_symmetry_tool(self):
        cmd = {
            'cmd': 'open_symmetry_tool'
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def detach_components(self, components=None):
        cmd = {
            'cmd': 'detach_components',
            'components': components
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def curve_from_edge_loop(self, mesh_edge_list=None):
        cmd = {
            'cmd': 'curve_from_edge_loop',
            'mesh_edge_list': mesh_edge_list
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['result']

    # =================================================================================================================
    # MAYA - JOINTS
    # =================================================================================================================

    def start_joint_tool(self):
        cmd = {
            'cmd': 'start_joint_tool'
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def create_new_joint_on_center(self, transforms=None):
        cmd = {
            'cmd': 'create_new_joint_on_center',
            'transforms': transforms
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def create_new_joints_on_selected_components(self):
        cmd = {
            'cmd': 'create_new_joints_on_selected_components'
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def insert_joints(self, joints=None, num_joints=1):
        cmd = {
            'cmd': 'insert_joints',
            'joints': joints,
            'num_joints': num_joints
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def create_joints_on_curve(self, curve=None, num_joints=1):
        cmd = {
            'cmd': 'create_joints_on_curve',
            'curve': curve,
            'num_joints': num_joints
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def snap_joints_to_curve(self, joints=None, curve=None, num_joints=1):
        cmd = {
            'cmd': 'snap_joints_to_curve',
            'joints': joints,
            'curve': curve,
            'num_joints': num_joints
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def toggle_local_rotation_axis(self, joints=None):
        cmd = {
            'cmd': 'toggle_local_rotation_axis',
            'joints': joints
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return False

        return reply_dict['success']

    def toggle_all_local_rotation_axis(self, flag=None):
        cmd = {
            'cmd': 'toggle_all_local_rotation_axis',
            'flag': flag
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return False

        return reply_dict['success']

    def toggle_selected_local_rotation_axis(self, flag=None):
        cmd = {
            'cmd': 'toggle_selected_local_rotation_axis',
            'flag': flag
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return False

        return reply_dict['success']

    def toggle_joints_xray(self):
        cmd = {
            'cmd': 'toggle_joints_xray'
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return False

        return reply_dict['success']

    def set_joints_xray(self, flag=None):
        cmd = {
            'cmd': 'set_joints_xray',
            'flag': flag
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return False

        return reply_dict['success']

    def set_joints_display_size(self, value=None):
        cmd = {
            'cmd': 'set_joints_display_size',
            'value': value
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return False

        return reply_dict['success']

    def select_hierarchy(self):
        cmd = {
            'cmd': 'select_hierarchy'
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return False

        return reply_dict['success']

    def orient_joints(self):
        cmd = {
            'cmd': 'orient_joints'
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return False

        return reply_dict['success']

    def orient_all_joints(self):
        cmd = {
            'cmd': 'orient_all_joints'
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return False

        return reply_dict['success']

    def orient_selected_joints(self):
        cmd = {
            'cmd': 'orient_selected_joints'
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return False

        return reply_dict['success']

    def add_orient_data(self):
        cmd = {
            'cmd': 'add_orient_data'
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return False

        return reply_dict['success']

    def add_orient_data_all_joints(self):
        cmd = {
            'cmd': 'add_orient_data_all_joints'
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return False

        return reply_dict['success']

    def add_orient_data_selected_joints(self):
        cmd = {
            'cmd': 'add_orient_data_selected_joints'
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return False

        return reply_dict['success']

    def clean_orient_data(self):
        cmd = {
            'cmd': 'clean_orient_data'
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return False

        return reply_dict['success']

    def clean_orient_data_all_joints(self):
        cmd = {
            'cmd': 'clean_orient_data_all_joints'
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return False

        return reply_dict['success']

    def clean_orient_data_selected_joints(self):
        cmd = {
            'cmd': 'clean_orient_data_selected_joints'
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return False

        return reply_dict['success']

    def zero_joint_orient(self):
        cmd = {
            'cmd': 'zero_joint_orient'
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return False

        return reply_dict['success']

    def zero_joint_orient_all_joints(self):
        cmd = {
            'cmd': 'zero_joint_orient_all_joints'
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return False

        return reply_dict['success']

    def zero_joint_orient_selected_joints(self):
        cmd = {
            'cmd': 'zero_joint_orient_selected_joints'
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return False

        return reply_dict['success']

    def orient_tool(self):

        cmd = {
            'cmd': 'setup_orient_joint_client'
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return False

        # Once client is up and running in DCC, we can launch the tool
        tools.ToolsManager().launch_tool_by_id('tpRigToolkit-tools-jointorient')

        return reply_dict['success']

    def mirror_joints(self):

        cmd = {
            'cmd': 'mirror_joints'
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return False

        return reply_dict['success']

    def mirror_all_joints(self):

        cmd = {
            'cmd': 'mirror_all_joints'
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return False

        return reply_dict['success']

    def mirror_selected_joints(self):

        cmd = {
            'cmd': 'mirror_selected_joints'
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return False

        return reply_dict['success']

    def mirror_hierarchy_joints(self):

        cmd = {
            'cmd': 'mirror_hierarchy_joints'
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return False

        return reply_dict['success']

    # =================================================================================================================
    # MAYA - SKINNING
    # =================================================================================================================

    def smooth_bind_skin(self, geo_node=None, show_options=False):
        cmd = {
            'cmd': 'smooth_bind_skin',
            'geo_node': geo_node,
            'show_options': show_options
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def rigid_bind_skin(self, geo_node=None, show_options=False):
        cmd = {
            'cmd': 'rigid_bind_skin',
            'geo_node': geo_node,
            'show_options': show_options
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def detach_bind_skin(self, geo_node=None, show_options=False):
        cmd = {
            'cmd': 'detach_bind_skin',
            'geo_node': geo_node,
            'show_options': show_options
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def open_paint_skin_weights_tool(self, show_options=False):
        cmd = {
            'cmd': 'open_paint_skin_weights_tool',
            'show_options': show_options
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def check_joints_labels(self, joints=None):
        cmd = {
            'cmd': 'check_joints_labels',
            'joints': joints
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['result']

    def mirror_skin_weights(
            self, mesh=None, auto_assign_labels=True, left_side_label='*_l_*', right_side_label='*_r_*',
            show_options=False):
        cmd = {
            'cmd': 'mirror_skin_weights',
            'mesh': mesh,
            'auto_assign_labels': auto_assign_labels,
            'left_side_label': left_side_label,
            'right_side_label': right_side_label,
            'show_options': show_options
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def copy_skin_weights(
            self, source_mesh=None, target_mesh=None, auto_assign_labels=True, left_side_label='*_l_*',
            right_side_label='*_r_*', show_options=False):
        cmd = {
            'cmd': 'copy_skin_weights',
            'source_mesh': source_mesh,
            'target_mesh': target_mesh,
            'auto_assign_labels': auto_assign_labels,
            'left_side_label': left_side_label,
            'right_side_label': right_side_label,
            'show_options': show_options
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def prune_skin_weights(self, mesh=None, show_options=False):
        cmd = {
            'cmd': 'prune_skin_weights',
            'mesh': mesh,
            'show_options': show_options
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def transfer_skin_uvs(
            self, source_mesh=None, target_mesh=None, auto_assign_labels=True, left_side_label='*_l_*',
            right_side_label='*_r_*'):
        cmd = {
            'cmd': 'transfer_skin_uvs',
            'source_mesh': source_mesh,
            'target_mesh': target_mesh,
            'auto_assign_labels': auto_assign_labels,
            'left_side_label': left_side_label,
            'right_side_label': right_side_label
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def clean_skinned_mesh(self, mesh=None, auto_assign_labels=True, left_side_label='*_l_*', right_side_label='*_r_*'):
        cmd = {
            'cmd': 'clean_skinned_mesh',
            'mesh': mesh,
            'auto_assign_labels': auto_assign_labels,
            'left_side_label': left_side_label,
            'right_side_label': right_side_label
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def combine_skinned_meshes(self, meshes=None):
        cmd = {
            'cmd': 'combine_skinned_meshes',
            'meshes': meshes
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['result']

    def extract_skinned_selected_faces(
            self, components=None, auto_assign_labels=True, left_side_label='*_l_*', right_side_label='*_r_*'):
        cmd = {
            'cmd': 'extract_skinned_selected_faces',
            'components': components,
            'auto_assign_labels': auto_assign_labels,
            'left_side_label': left_side_label,
            'right_side_label': right_side_label
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['result']

    def remove_unused_influences(self, mesh=None):
        cmd = {
            'cmd': 'remove_unused_influences',
            'mesh': mesh
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def restore_bind_pose(self, mesh=None):
        cmd = {
            'cmd': 'restore_bind_pose',
            'mesh': mesh
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def remove_bind_poses(self):
        cmd = {
            'cmd': 'remove_bind_poses'
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def weights_hammer(self, vertices_to_hammer=None):
        cmd = {
            'cmd': 'weights_hammer',
            'vertices_to_hammer': vertices_to_hammer
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def load_plugins(self, do_reload=True):
        cmd = {
            'cmd': 'load_plugins',
            'do_reload': do_reload
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def average_vertex_weights(self, components=None, use_distance=True, curve_weight_points=None):
        cmd = {
            'cmd': 'average_vertex_weights',
            'components': components,
            'use_distance': use_distance,
            'curve_weight_points': curve_weight_points
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def move_skin_weights(self):
        cmd = {
            'cmd': 'move_skin_weights'
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def swap_skin_weights(self):
        cmd = {
            'cmd': 'swap_skin_weights'
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def select_influences(self, mesh_node=None):
        cmd = {
            'cmd': 'select_influences',
            'mesh_node': mesh_node

        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def select_influence_components(self, influence_joints=None, mesh_node=None):
        cmd = {
            'cmd': 'select_influence_components',
            'influence_joints': influence_joints,
            'mesh_node': mesh_node
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def delete_influences(self, mesh_nodes=None, influences_to_remove=None, fast_delete=True):
        cmd = {
            'cmd': 'delete_influences',
            'mesh_nodes': mesh_nodes,
            'influences_to_remove': influences_to_remove,
            'fast_delete': fast_delete
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def br_smooth_weights(self, options=False):
        cmd = {
            'cmd': 'br_smooth_weights',
            'options': options
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def br_transfer_weights(self, options=False):
        cmd = {
            'cmd': 'br_transfer_weights',
            'options': options
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']

    def ng_skin_tools(self):
        cmd = {
            'cmd': 'ng_skin_tools'
        }

        reply_dict = self.send(cmd)

        if not self.is_valid_reply(reply_dict):
            return list()

        return reply_dict['success']



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains widget for rig control functionality
"""

from __future__ import print_function, division, absolute_import

from tpRigToolkit.tools.rigtoolbox.widgets import base

from tpRigToolkit.tools.controlrig.core import client, tool, toolset


class ControlRigToolset(toolset.ControlRigToolset):
    def __init__(self, *args, **kwargs):
        super(ControlRigToolset, self).__init__(*args, **kwargs)

        self._control_rig_client = client.ControlRigClient.create_and_connect_to_server(tool.ControlRigTool.ID)
        self.initialize(client=self._control_rig_client)
        self._title_frame.setVisible(False)


class ControlWidget(base.BaseRigToolBoxWidget, object):
    def __init__(self, parent=None):
        super(ControlWidget, self).__init__(title='Controls', parent=parent)

        ctrl_rig = ControlRigToolset(collapsable=False, parent=self)
        self.main_layout.addWidget(ctrl_rig)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains widget for symmesh functionality
"""

from __future__ import print_function, division, absolute_import

from tpRigToolkit.tools.symmesh.core import client, tool, toolset
from tpRigToolkit.tools.rigtoolbox.widgets import base


class SymmeshToolset(toolset.SymMeshToolset):
    def __init__(self, *args, **kwargs):
        super(SymmeshToolset, self).__init__(*args, **kwargs)

        self._symmesh_client = client.SymmeshClient.create_and_connect_to_server(tool.SymMeshTool.ID)
        self.initialize(client=self._symmesh_client)
        self._title_frame.setVisible(False)


class SymmeshWidget(base.BaseRigToolBoxWidget, object):
    def __init__(self, parent=None):
        super(SymmeshWidget, self).__init__(title='SymMesh', parent=parent)

        rename_widget = SymmeshToolset(parent=self)
        self.main_layout.addWidget(rename_widget)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Init module of RigToolbox for Maya
"""

from __future__ import print_function, division, absolute_import

import logging
import traceback

from tpRigToolkit.tools.rigtoolbox.core import consts

LOGGER = logging.getLogger(consts.TOOL_ID)


def get_toolbox_widgets(client, commands_data, parent=None):

    all_widgets = list()
    try:
        from tpRigToolkit.tools.rigtoolbox.dccs.maya.widgets import general
        general_commands_data = commands_data.get('general', dict())
        if commands_data:
            general_widget = general.GeneralWidget(commands_data=general_commands_data, client=client, parent=parent)
            all_widgets.append(general_widget)
        else:
            LOGGER.warning('General widget not loaded because not commands data found!')
    except Exception:
        LOGGER.exception('Error while creating general widget: "{}"'.format(traceback.format_exc()))

    try:
        from tpRigToolkit.tools.rigtoolbox.dccs.maya.widgets import joint
        joint_commmand_data = commands_data.get('joint', dict())
        if joint_commmand_data:
            joint_widget = joint.JointWidget(commands_data=joint_commmand_data, client=client, parent=parent)
            all_widgets.append(joint_widget)
    except Exception:
        LOGGER.exception('Error while creating joint widget: "{}"'.format(traceback.format_exc()))

    try:
        from tpRigToolkit.tools.rigtoolbox.dccs.maya.widgets import skin
        skin_commmand_data = commands_data.get('skin', dict())
        skinning_widget = skin.SkinningWidget(commands_data=skin_commmand_data, client=client, parent=parent)
        skinning_widget.refresh()
        all_widgets.append(skinning_widget)
    except Exception:
        LOGGER.exception('Error while creating skinning widget: "{}"'.format(traceback.format_exc()))

    try:
        from tpRigToolkit.tools.rigtoolbox.dccs.maya.widgets import rename
        client.setup_renamer_client()
        rename_widget = rename.RenameWidget()
        rename_widget.setParent(parent)
        all_widgets.append(rename_widget)
    except Exception:
        LOGGER.exception('Error while creating renamer widget: "{}"'.format(traceback.format_exc()))

    try:
        from tpRigToolkit.tools.rigtoolbox.dccs.maya.widgets import control
        client.setup_control_rig_client()
        control_widget = control.ControlWidget()
        control_widget.setParent(parent)
        all_widgets.append(control_widget)
    except Exception:
        LOGGER.exception('Error while creating control widget: "{}"'.format(traceback.format_exc()))

    try:
        from tpRigToolkit.tools.rigtoolbox.dccs.maya.widgets import symmesh
        client.setup_symmesh_client()
        symmesh_widget = symmesh.SymmeshWidget()
        symmesh_widget.setParent(parent)
        all_widgets.append(symmesh_widget)
    except Exception:
        LOGGER.exception('Error while creating symmesh widget: "{}"'.format(traceback.format_exc()))

    return all_widgets



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains rig toolbox server implementation for Maya
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"

import os
import traceback

from tpDcc import dcc
from tpDcc.core import server
from tpDcc.libs.python import path as path_utils
from tpDcc.dccs.maya.core import helpers

from tpDcc.tools.renamer.core import tool as renamer_tool
from tpDcc.tools.renamer.dccs.maya import server as renamer_server


from tpRigToolkit.tools.controlrig.core import tool as controlrig_tool
from tpRigToolkit.tools.controlrig.dccs.maya import server as controlrig_server
from tpRigToolkit.tools.jointorient.core import tool as jointorient_tool
from tpRigToolkit.tools.jointorient.dccs.maya import server as jointorient_server
from tpRigToolkit.tools.symmesh.core import tool as symmesh_tool
from tpRigToolkit.tools.symmesh.dccs.maya import server as symmesh_server

from tpRigToolkit.tools.rigtoolbox.dccs.maya.libs import general, joint, skin


class RigToolboxServer(server.DccServer, object):

    PORT = 19344

    def __init__(self, *args, **kwargs):
        super(RigToolboxServer, self).__init__(*args, **kwargs)

        self._renamer_client = None
        self._control_rig_client = None
        self._orient_joint_client = None
        self._symmesh_client = None

    # =================================================================================================================
    # RENAMER
    # =================================================================================================================

    def setup_renamer_client(self, data, reply):
        if not self._renamer_client:
            self._renamer_client = dcc.client(renamer_tool.RenamerTool.ID)
        try:
            try:
                self._renamer_client.close_connection()
            except Exception:
                pass
            self._renamer_client = renamer_server.RenamerServer()
        except Exception:
            print('Error while setting up renamer server: '.format(traceback.format_exc()))

        reply['success'] = True

    def setup_control_rig_client(self, data, reply):
        if not self._control_rig_client:
            self._control_rig_client = dcc.client(controlrig_tool.ControlRigTool.ID)
        try:
            try:
                self._control_rig_client.close_connection()
            except Exception:
                pass
            self._control_rig_client = controlrig_server.ControlRigServer()
        except Exception:
            print('Error while setting up control rig server: '.format(traceback.format_exc()))

        reply['success'] = True

    def setup_orient_joint_client(self, data, reply):
        if not self._orient_joint_client:
            self._orient_joint_client = dcc.client(jointorient_tool.JointOrientTool.ID)
        try:
            try:
                self._orient_joint_client.close_connection()
            except Exception:
                pass
            self._orient_joint_client = jointorient_server.JointOrientServer()
        except Exception:
            print('Error while setting up joint orient server: '.format(traceback.format_exc()))

        reply['success'] = True

    def setup_symmesh_client(self, data, reply):
        if not self._symmesh_client:
            self._symmesh_client = dcc.client(symmesh_tool.SymMeshTool.ID)
        try:
            try:
                self._symmesh_client.close_connection()
            except Exception:
                pass
            self._symmesh_client = symmesh_server.SymmeshServer()
        except Exception:
            print('Error while setting up symmesh server: '.format(traceback.format_exc()))

        reply['success'] = True

    # =================================================================================================================
    # GENERAL
    # =================================================================================================================

    def delete_history(self, data, reply):

        transforms = data['transforms']

        try:
            result = general.delete_history(transforms)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while deleting history: {}'.format(traceback.format_exc())
            reply['success'] = False

    def freeze_transforms(self, data, reply):

        transforms = data['transforms']

        try:
            result = general.freeze_transforms(transforms)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while freezing transforms: {}'.format(traceback.format_exc())
            reply['success'] = False

    def move_pivot_to_zero(self, data, reply):
        transforms = data['transforms']

        try:
            result = general.move_pivot_to_zero(transforms)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while moving pivots to zero: {}'.format(traceback.format_exc())
            reply['success'] = False

    def lock_all_transforms(self, data, reply):
        transforms = data['transforms']

        try:
            result = general.lock_all_transforms(transforms)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while locking all transforms channels: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    def lock_translation(self, data, reply):
        transforms = data['transforms']

        try:
            result = general.lock_translation(transforms)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while locking translate channels: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    def lock_rotation(self, data, reply):
        transforms = data['transforms']

        try:
            result = general.lock_rotation(transforms)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while locking rotate channels: {}'.format(traceback.format_exc())
            reply['success'] = False

    def lock_scale(self, data, reply):
        transforms = data['transforms']

        try:
            result = general.lock_scale(transforms)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while locking scale channel: {}'.format(traceback.format_exc())
            reply['success'] = False

    def lock_visibility(self, data, reply):
        transforms = data['transforms']

        try:
            result = general.lock_visibility(transforms)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while locking visibility channel: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    def unlock_all_transforms(self, data, reply):
        transforms = data['transforms']

        try:
            result = general.unlock_all_transforms(transforms)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while unlocking all transforms channels: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    def unlock_translation(self, data, reply):
        transforms = data['transforms']

        try:
            result = general.unlock_translation(transforms)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while unlocking translate channels: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    def unlock_rotation(self, data, reply):
        transforms = data['transforms']

        try:
            result = general.unlock_rotation(transforms)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while unlocking rotate channels: {}'.format(traceback.format_exc())
            reply['success'] = False

    def unlock_scale(self, data, reply):
        transforms = data['transforms']

        try:
            result = general.unlock_scale(transforms)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while unlocking scale channel: {}'.format(traceback.format_exc())
            reply['success'] = False

    def unlock_visibility(self, data, reply):
        transforms = data['transforms']

        try:
            result = general.unlock_visibility(transforms)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while unlocking visibility channel: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    def clean_student_license(self, data, reply):
        file_path = data.get('file_path', None)

        try:
            helpers.clean_student_line(file_path)
            reply['success'] = True
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while cleaning student license: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    def match_transform(self, data, reply):
        source_transform = data['source_transform']
        target_transform = data['target_transform']

        try:
            result = general.match_transform(source_transform, target_transform)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while matching transforms: {}'.format(traceback.format_exc())
            reply['success'] = False

    def match_translation(self, data, reply):
        source_transform = data['source_transform']
        target_transform = data['target_transform']

        try:
            result = general.match_translation(source_transform, target_transform)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while matching translation: {}'.format(traceback.format_exc())
            reply['success'] = False

    def match_rotation(self, data, reply):
        source_transform = data['source_transform']
        target_transform = data['target_transform']

        try:
            result = general.match_rotation(source_transform, target_transform)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while matching rotation: {}'.format(traceback.format_exc())
            reply['success'] = False

    def match_scale(self, data, reply):
        source_transform = data['source_transform']
        target_transform = data['target_transform']

        try:
            result = general.match_scale(source_transform, target_transform)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while matching scale: {}'.format(traceback.format_exc())
            reply['success'] = False

    def combine_meshes(self, data, reply):
        meshes = data['meshes']
        new_mesh_name = data['new_mesh_name']

        try:
            result = general.combine_meshes(meshes, new_mesh_name)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while combining meshes: {}'.format(traceback.format_exc())
            reply['success'] = False

    def separate_meshes(self, data, reply):
        meshes = data['meshes']

        try:
            result = general.separate_meshes(meshes)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while separating meshes: {}'.format(traceback.format_exc())
            reply['success'] = False

    def mirror_meshes(self, data, reply):
        meshes = data['meshes']

        try:
            result = general.mirror_mesh(meshes)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while mirroring mesh: {}'.format(traceback.format_exc())
            reply['success'] = False

    def mirror_meshes_options(self, data, reply):
        try:
            result = general.open_mirror_meshes_options()
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while opening mirror mesh options: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    def open_symmetry_tool(self, data, reply):
        try:
            result = general.open_symmetry_tool()
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while opening symmetry tool: {}'.format(traceback.format_exc())
            reply['success'] = False

    def detach_components(self, data, reply):
        components = data['components']

        try:
            result = general.detach_components(components)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while detaching components: {}'.format(traceback.format_exc())
            reply['success'] = False

    def curve_from_edge_loop(self, data, reply):
        mesh_edge_list = data['mesh_edge_list']

        try:
            result = general.create_curve_from_mesh_edge_loop(mesh_edge_list)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while creating curve from edge loop: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    # =================================================================================================================
    # MAYA - JOINTS
    # =================================================================================================================

    def start_joint_tool(self, data, reply):
        try:
            result = joint.start_joint_tool()
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while opening joint tool: {}'.format(traceback.format_exc())
            reply['success'] = False

    def create_new_joint_on_center(self, data, reply):
        transforms = data['transforms']

        try:
            result = joint.create_new_joint_on_center(transforms=transforms)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while creating new ' \
                               'joints on the center of transforms: {}'.format(traceback.format_exc())
            reply['success'] = False

    def create_new_joints_on_selected_components(self, data, reply):
        try:
            result = joint.create_joints_on_selected_components_center()
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while creating new ' \
                               'joints on the center of selected components: {}'.format(traceback.format_exc())
            reply['success'] = False

    def insert_joints(self, data, reply):
        joints = data['joints']
        num_joints = data['num_joints']

        try:
            result = joint.insert_joints(joints, num_joints)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while inserting joints: {}'.format(traceback.format_exc())
            reply['success'] = False

    def create_joints_on_curve(self, data, reply):
        curve = data['curve']
        num_joints = data['num_joints']

        try:
            result = joint.create_joints_on_curve(curve, num_joints)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while creating joints on curve: {}'.format(traceback.format_exc())
            reply['success'] = False

    def snap_joints_to_curve(self, data, reply):
        joints = data['joints']
        curve = data['curve']
        num_joints = data['num_joints']

        try:
            result = joint.snap_joints_to_curve(joints, curve, num_joints)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while snapping joints to curve: {}'.format(traceback.format_exc())
            reply['success'] = False

    def toggle_local_rotation_axis(self, data, reply):
        joints = data['joints']

        try:
            result = joint.toggle_local_rotation_axis(joints)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while toggling joints LRA: {}'.format(traceback.format_exc())
            reply['success'] = False

    def toggle_all_local_rotation_axis(self, data, reply):

        flag = data.get('flag', None)

        try:
            result = joint.toggle_all_local_rotation_axis(flag=flag)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while toggling all joints LRA: {}'.format(traceback.format_exc())
            reply['success'] = False

    def toggle_selected_local_rotation_axis(self, data, reply):

        flag = data.get('flag', None)

        try:
            result = joint.toggle_selected_local_rotation_axis(flag=flag)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while toggling selected joints LRA: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    def toggle_joints_xray(self, data, reply):
        try:
            result = joint.toggle_joints_xray()
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while toggling joints X-Ray: {}'.format(traceback.format_exc())
            reply['success'] = False

    def set_joints_xray(self, data, reply):
        flag = data.get('flag', None)

        try:
            result = joint.set_joints_xray(flag=flag)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while setting joints X-Ray: {}'.format(traceback.format_exc())
            reply['success'] = False

    def set_joints_display_size(self, data, reply):
        value = data.get('value', None)

        try:
            result = joint.set_joints_display_size(value=value)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while setting joints display size: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    def select_hierarchy(self, data, reply):
        try:
            result = joint.select_hierarchy()
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while selecting joints hierarchy: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    def orient_joints(self, data, reply):
        try:
            result = joint.orient_joints()
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while orienting joints: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    def orient_all_joints(self, data, reply):
        try:
            result = joint.orient_all_joints()
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while orienting all scene joints: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    def orient_selected_joints(self, data, reply):
        try:
            result = joint.orient_selected_joints()
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while orienting selected joints: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    def add_orient_data(self, data, reply):
        try:
            result = joint.add_orient_data()
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while adding orient data joints: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    def add_orient_data_all_joints(self, data, reply):
        try:
            result = joint.orient_all_joints()
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while adding orient data to all scene joints: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    def add_orient_data_selected_joints(self, data, reply):
        try:
            result = joint.orient_selected_joints()
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while adding orient data to selected joints: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    def clean_orient_data(self, data, reply):
        try:
            result = joint.clean_orient_data()
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while cleaning orient data from joints: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    def clean_orient_data_all_joints(self, data, reply):
        try:
            result = joint.clean_orient_data_all_joints()
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while cleaning orient data from scene joints: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    def clean_orient_data_selected_joints(self, data, reply):
        try:
            result = joint.clean_orient_data_selected_joints()
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while cleaning orient data from selected joints: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    def zero_joint_orient(self, data, reply):
        try:
            result = joint.zero_joint_orient()
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while zeroing out orient of joints: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    def zero_joint_orient_all_joints(self, data, reply):
        try:
            result = joint.zero_joint_orient_all_joints()
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something wrong while zeroing out orient of all joints in current scene: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    def zero_joint_orient_selected_joints(self, data, reply):
        try:
            result = joint.zero_joint_orient_selected_joints()
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while zeroing out orient of selected joints: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    def mirror_joints(self, data, reply):
        try:
            result = joint.mirror_joints()
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while mirroring joints: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    def mirror_all_joints(self, data, reply):
        try:
            result = joint.mirror_all_joints()
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while mirroring all joints in current scene: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    def mirror_selected_joints(self, data, reply):
        try:
            result = joint.mirror_selected_joints()
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while mirroring selected joints: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    def mirror_hierarchy_joints(self, data, reply):
        try:
            result = joint.mirror_hierarchy_joints()
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while mirroring hierarchy joints: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    # =================================================================================================================
    # SKINNING
    # =================================================================================================================

    def smooth_bind_skin(self, data, reply):

        geo_node = data['geo_node']
        show_options = data['show_options']

        try:
            result = skin.apply_smooth_bind_skin(geo=geo_node, show_options=show_options)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while applying smooth bind skin: {}'.format(traceback.format_exc())
            reply['success'] = False

    def rigid_bind_skin(self, data, reply):

        geo_node = data['geo_node']
        show_options = data['show_options']

        try:
            result = skin.apply_rigid_skin(geo=geo_node, show_options=show_options)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while applying rigid bind skin: {}'.format(traceback.format_exc())
            reply['success'] = False

    def detach_bind_skin(self, data, reply):

        geo_node = data['geo_node']
        show_options = data['show_options']

        try:
            result = skin.detach_bind_skin(geo=geo_node, show_options=show_options)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while detaching bind skin: {}'.format(traceback.format_exc())
            reply['success'] = False

    def open_paint_skin_weights_tool(self, data, reply):

        show_options = data['show_options']

        try:
            result = skin.open_pain_skin_weights_tool(show_options=show_options)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while opening skin weights paint tool: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    def check_joints_labels(self, data, reply):

        joints = data['joints']

        try:
            result = joint.check_joint_labels(joints=joints)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while checking joints label: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    def mirror_skin_weights(self, data, reply):

        mesh = data['mesh']
        auto_assign_labels = data['auto_assign_labels']
        left_side_label = data['left_side_label']
        right_side_label = data['right_side_label']
        show_options = data['show_options']

        try:
            result = skin.mirror_skin_weights(
                mesh=mesh, auto_assign_labels=auto_assign_labels, left_side_label=left_side_label,
                right_side_label=right_side_label, show_options=show_options)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while mirroring skin weights: {}'.format(traceback.format_exc())
            reply['success'] = False

    def copy_skin_weights(self, data, reply):

        source_mesh = data['source_mesh']
        target_mesh = data['target_mesh']
        auto_assign_labels = data['auto_assign_labels']
        left_side_label = data['left_side_label']
        right_side_label = data['right_side_label']
        show_options = data['show_options']

        try:
            result = skin.copy_skin_weights(
                source_mesh=source_mesh, target_mesh=target_mesh, auto_assign_labels=auto_assign_labels,
                left_side_label=left_side_label, right_side_label=right_side_label, show_options=show_options)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while copying skin weights: {}'.format(traceback.format_exc())
            reply['success'] = False

    def prune_skin_weights(self, data, reply):
        show_options = data['show_options']
        mesh = data['mesh']

        try:
            result = skin.prune_skin_weights(geo=mesh, show_options=show_options)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while pruning skin weights: {}'.format(traceback.format_exc())
            reply['success'] = False

    def transfer_skin_uvs(self, data, reply):

        source_mesh = data['source_mesh']
        target_mesh = data['target_mesh']
        auto_assign_labels = data['auto_assign_labels']
        left_side_label = data['left_side_label']
        right_side_label = data['right_side_label']

        try:
            result = skin.transfer_uvs_to_skinned_geometry(
                source_mesh=source_mesh, target_mesh=target_mesh, auto_assign_labels=auto_assign_labels,
                left_side_label=left_side_label, right_side_label=right_side_label)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while transferring UVs to skinned geometry: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    def clean_skinned_mesh(self, data, reply):
        mesh = data['mesh']
        auto_assign_labels = data['auto_assign_labels']
        left_side_label = data['left_side_label']
        right_side_label = data['right_side_label']

        try:
            result = skin.freeze_skinned_mesh(
                mesh, auto_assign_labels=auto_assign_labels, left_side_label=left_side_label,
                right_side_label=right_side_label)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while cleaning skinned geometry: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    def combine_skinned_meshes(self, data, reply):
        meshes = data['meshes']

        try:
            result = skin.combine_skinned_meshes(meshes)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while combining skinned meshes: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    def extract_skinned_selected_faces(self, data, reply):
        components = data['components']
        auto_assign_labels = data['auto_assign_labels']
        left_side_label = data['left_side_label']
        right_side_label = data['right_side_label']

        try:
            result = skin.extract_skinned_selected_components(
                components, auto_assign_labels=auto_assign_labels, left_side_label=left_side_label,
                right_side_label=right_side_label)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while extracting skinned selected faces: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    def remove_unused_influences(self, data, reply):
        mesh = data['mesh']

        try:
            result = skin.delete_unused_influences(mesh)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while removing unused influences: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    def restore_bind_pose(self, data, reply):
        mesh = data['mesh']

        try:
            result = skin.restore_to_bind_pose(mesh)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while restoring bind pose: {}'.format(traceback.format_exc())
            reply['success'] = False

    def remove_bind_poses(self, data, reply):

        try:
            result = skin.remove_bind_poses()
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while removing all bind poses from current scene: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    def weights_hammer(self, data, reply):

        vertices_to_hammer = data['vertices_to_hammer']

        try:
            result = skin.hammer_vertices(vertices_to_hammer)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while hammering weights: {}'.format(
                    traceback.format_exc())
            reply['success'] = False

    def average_vertex_weights(self, data, reply):
        components = data['components'] or dcc.selected_nodes_in_order(flatten=True)
        use_distance = data['use_distance']
        curve_weight_points = data['curve_weight_points']

        try:
            result = skin.average_vertices_weights(
                components, use_distance=use_distance, curve_weight_points=curve_weight_points)
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while averaging vertex weights: {}'.format(traceback.format_exc())
            reply['success'] = False

    def move_skin_weights(self, data, reply):

        try:
            result = skin.move_skin_weights()
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while moving skinning weights: {}'.format(traceback.format_exc())
            reply['success'] = False

    def swap_skin_weights(self, data, reply):

        try:
            result = skin.swap_skin_weights()
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while moving skinning weights: {}'.format(traceback.format_exc())
            reply['success'] = False

    def select_influences(self, data, reply):

        try:
            result = skin.select_influencing_joints()
            reply.update(result)
        except Exception:
            if not reply['msg']:
                reply['msg'] = 'Something went wrong while selecting influences {}'.format(traceback.format_exc())
            reply['success'] = False

    def select_influence_components(self, data, reply):

        influence_joints = data['influence_joints']
        mesh_node = data['mesh_node']

        success = skin.select_influence_components(joint_nodes=influence_joints, mesh_name=mesh_node)

        reply['success'] = success

    def delete_influences(self, data, reply):

        mesh_nodes = data['mesh_nodes']
        influences_to_remove = data['influences_to_remove']
        fast_delete = data['fast_delete']

        success = skin.delete_influences(
            skinned_objects=mesh_nodes, influences_to_remove=influences_to_remove, fast=fast_delete)

        reply['success'] = success

    def br_smooth_weights(self, data, reply):

        options = data.get('options', False)

        success = skin.br_smooth_weights_options() if options else skin.br_smooth_weights()

        reply['success'] = success

    def br_transfer_weights(self, data, reply):

        options = data.get('options', False)

        success = skin.br_smooth_weights_options() if options else skin.br_transfer_weights()

        reply['success'] = success

    def ng_skin_tools(self, data, reply):

        success = skin.ng_skin_tools()

        reply['success'] = success

    def load_plugins(self, data, reply):

        do_reload = data['do_reload']

        plugins_path = path_utils.clean_path(os.path.join(os.path.abspath(os.path.dirname(__file__)), 'plugins'))
        if not os.path.isdir(plugins_path):
            reply['msg'] = 'Impossible to load tpRigToolkit-tools-rigtoolbox Maya plugins because plugin ' \
                           'folder was not found: "{}"'.format(plugins_path)
            reply['success'] = False
            return

        # Update Maya Plugin paths
        maya_plugin_path = os.getenv('MAYA_PLUG_IN_PATH', None)
        if not maya_plugin_path:
            os.environ['MAYA_PLUG_IN_PATH'] = plugins_path
        else:
            path_already_added = False
            current_plugin_paths = os.environ['MAYA_PLUG_IN_PATH'].split(os.pathsep)
            for current_plugin_path in current_plugin_paths:
                if path_utils.clean_path(current_plugin_path) == plugins_path:
                    path_already_added = True
                    break
            if not path_already_added:
                os.environ['MAYA_PLUG_IN_PATH'] = '{}{}{}'.format(
                    os.environ['MAYA_PLUG_IN_PATH'], os.pathsep, plugins_path)

        # Load plugins
        for plugin_file in os.listdir(plugins_path):
            if not plugin_file:
                continue
            plugin_ext = os.path.splitext(plugin_file)[-1]
            if not plugin_ext == '.py':
                continue
            plugin_path = path_utils.clean_path(os.path.join(plugins_path, plugin_file))
            if do_reload:
                if helpers.is_plugin_loaded(plugin_path):
                    helpers.unload_plugin(plugin_path)
            helpers.load_plugin(plugin_path)

        reply['success'] = True



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains widget for rename functionality
"""

from __future__ import print_function, division, absolute_import

from tpDcc.managers import configs
from tpDcc.tools.renamer.core import client, tool, toolset

from tpRigToolkit.tools.rigtoolbox.widgets import base


class RenameToolset(toolset.RenamerToolset):
    def __init__(self, *args, **kwargs):
        super(RenameToolset, self).__init__(*args, **kwargs)

        self._rename_client = client.RenamerClient.create_and_connect_to_server(tool.RenamerTool.ID)
        self.initialize(client=self._rename_client)
        self._title_frame.setVisible(False)


class RenameWidget(base.BaseRigToolBoxWidget, object):
    def __init__(self, dev=True, parent=None):
        super(RenameWidget, self).__init__(title='Rename', parent=parent)

        names_config = configs.get_config(
            config_name='tpRigToolkit-names', environment='development' if dev else 'production')
        naming_config = configs.get_config(
            config_name='tpRigToolkit-naming', environment='development' if dev else 'production')
        rename_widget = RenameToolset(
            names_config=names_config, naming_config=naming_config, parent=self)
        self.main_layout.addWidget(rename_widget)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Collection of rigging tools
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"

import os
import logging

from Qt.QtCore import Qt
from Qt.QtWidgets import QSizePolicy, QToolButton, QMainWindow

from tpDcc.managers import resources
from tpDcc.libs.python import yamlio
from tpDcc.libs.qt.core import base
from tpDcc.libs.qt.widgets import window, toolset, progressbar, dock, message, buttons

from tpRigToolkit.tools.rigtoolbox.core import consts
from tpRigToolkit.tools.rigtoolbox.widgets import dock, library, base as base_widgets

LOGGER = logging.getLogger(consts.TOOL_ID)


class RigToolboxToolset(toolset.ToolsetWidget, object):
    def __init__(self, *args, **kwargs):
        super(RigToolboxToolset, self).__init__(*args, **kwargs)

    def help_mode(self, flag):
        if not flag:
            return

        help_event = toolset.ToolsetHelpEvent(
            self._rig_toolbox_widget.main_layout, self._toolbox_widgets, self._help_widget)

        return help_event

    def contents(self):
        dcc_name, dcc_version, _ = self.client.get_dcc_info()

        self.client.load_plugins()

        commands_datas = dict()
        if dcc_name:
            commands_path = os.path.join(
                os.path.abspath(os.path.dirname(os.path.dirname(__file__))), 'dccs', dcc_name, 'commands')
            if os.path.isdir(commands_path):
                for command_file in os.listdir(commands_path):
                    if not command_file.endswith('.yaml'):
                        continue
                    commands_category = os.path.splitext(command_file)[0]
                    if commands_category in commands_datas:
                        continue
                    command_file_path = os.path.join(commands_path, command_file)
                    try:
                        commands_data = yamlio.read_file(command_file_path, maintain_order=True)
                    except Exception as exc:
                        LOGGER.error(
                            'Error while reading commands data from "{}" : {}'.format(command_file_path, exc))
                        continue
                    commands_datas[commands_category] = commands_data

        # Load widgets
        self._rig_toolbox_widget = RigToolboxWidget(parent=self)
        self._toolbox_widgets = list()
        if not dcc_name:
            from tpRigToolkit.tools.rigtoolbox.widgets import base
            toolbox_widget = base.BaseRigToolBoxWidget(title='Hello World!', parent=self)
            self._toolbox_widgets.append(toolbox_widget)

        if self.client.is_maya():
            from tpRigToolkit.tools.rigtoolbox.dccs import maya
            self._toolbox_widgets = maya.get_toolbox_widgets(
                client=self.client, commands_data=commands_datas, parent=self)
        self._rig_toolbox_widget.load_widgets(self._toolbox_widgets, parent=self)

        library.Command.startCommand.connect(self._rig_toolbox_widget._on_start_command)
        library.Command.progressCommand.connect(self._rig_toolbox_widget._on_set_progress_command)
        library.Command.endCommand.connect(self._rig_toolbox_widget._on_end_command)
        # self.helpModeChanged.connect(rig_toolbox_widget.set_info_mode)

        return [self._rig_toolbox_widget]

    def reload_theme(self, theme=None):
        super(RigToolboxToolset, self).reload_theme(theme)

        if not theme:
            return
        stylesheet = theme.stylesheet()
        self._rig_toolbox_widget._base_window.setStyleSheet(stylesheet)


class RigToolboxWidget(base.BaseWidget, object):

    WindowName = 'RigToolBoxWidgetsWindow'

    def __init__(self, parent=None):
        super(RigToolboxWidget, self).__init__(parent=parent)

    def ui(self):
        super(RigToolboxWidget, self).ui()

        self._base_window = window.BaseWindow(parent=self)
        self._base_window.centralWidget().hide()
        self._base_window.setDockOptions(
            QMainWindow.AnimatedDocks | QMainWindow.AllowNestedDocks | QMainWindow.AllowTabbedDocks)
        self._base_window.setWindowFlags(Qt.Widget)

        self._progress = progressbar.BaseProgressBar(parent=self)
        self._progress.setMinimum(0)
        self._progress.setMaximum(100)

        self.main_layout.addWidget(self._base_window)
        self.main_layout.addWidget(self._progress)

    def load_widgets(self, widgets, parent=None):
        for w in widgets:
            if isinstance(w, base_widgets.BaseRigToolBoxWidget):
                w.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
                w.emitInfo.connect(self._on_show_success)
                w.emitError.connect(self._on_show_error)
                dock_widget = dock.DockWidget(w.title, parent=parent or self)
                info_button = QToolButton(parent=parent or self)
                info_button.setCheckable(True)
                info_button.setChecked(True)
                info_button.setIcon(resources.icon('info', theme='color'))
                if hasattr(w, '_on_toggle_info'):
                    info_button.toggled.connect(w._on_toggle_info)
                dock_widget.add_button(info_button)
            else:
                dock_widget = dock.DockWidget(w.objectName(), parent=parent or self)

            dock_widget.setWidget(w)
            self._base_window.addDockWidget(Qt.TopDockWidgetArea, dock_widget)

    def _on_start_command(self, command_name):
        library.Command.command = command_name
        self._progress.setFormat('Executing command: "{}"'.format(command_name))
        self._progress.setValue(0)
        self._progress.theme_status = progressbar.BaseProgressBar.NORMAL_STATUS

    def _on_set_progress_command(self, value, msg=None):
        current_command = library.Command.command
        if not current_command:
            return

        self._progress.setValue(value)
        if msg:
            self._progress.setFormat(msg)

    def _on_end_command(self, was_success):
        current_command = library.Command.command
        if not current_command:
            return
        if was_success:
            self._progress.setFormat('Command "{}" executed successfully!'.format(current_command))
            self._progress.theme_status = progressbar.BaseProgressBar.SUCCESS_STATUS
        else:
            self._progress.setFormat('Error while executing command "{}"!'.format(current_command))
            self._progress.theme_status = progressbar.BaseProgressBar.ERROR_STATUS
        self._progress.setValue(100)
        library.Command.command = None

    def _on_show_success(self, success_message):
        message.PopupMessage.success(success_message, self, closable=True)

    def _on_show_error(self, error_message):
        message.PopupMessage.error(error_message, self, closable=True)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains widget for rig skinning functionality
"""

from __future__ import print_function, division, absolute_import

import logging

from Qt.QtCore import Signal, QObject
from Qt.QtWidgets import QWidget, QMessageBox

from tpDcc.libs.qt.core import qtutils
from tpDcc.libs.qt.widgets import layouts, checkbox

from tpRigToolkit.tools.rigtoolbox.core import consts
from tpRigToolkit.tools.rigtoolbox.widgets import base, fallofcurve
from tpRigToolkit.tools.rigtoolbox.dccs.maya.widgets import labelsdialog

LOGGER = logging.getLogger(consts.TOOL_ID)


class SkinningWidget(base.CommandRigToolBoxWidget, object):
    def __init__(self, client, commands_data, parent=None):

        self._model = SkinningWidgetModel()

        super(SkinningWidget, self).__init__(
            title='Skinning', commands_data=commands_data,
            controller=SkinningWidgetController(model=self._model, client=client), parent=parent)

        self.refresh()

    @property
    def model(self):
        return self._model

    @property
    def controller(self):
        return self._controller

    def ui(self):
        super(SkinningWidget, self).ui()

        self._average_falloff_widget = QWidget()
        average_falloff_layout = layouts.VerticalLayout(spacing=2, margins=(5, 5, 5, 5))
        self._average_falloff_widget.setLayout(average_falloff_layout)
        self._average_falloff_curve = fallofcurve.FallofCurveWidget(parent=self)
        average_falloff_layout.addWidget(self._average_falloff_curve)

        self._mirror_auto_assign_joints_labels_cbx = checkbox.BaseCheckBox('Auto Assign Labels', self)
        self._copy_skin_weights_auto_assign_joints_labels_cbx = checkbox.BaseCheckBox('Auto Assign Labels', self)
        self._transfer_skin_uvs_auto_assign_joints_labels_cbx = checkbox.BaseCheckBox('Auto Assign Labels', self)
        self._clean_skin_mesh_auto_assign_joints_labels_cbx = checkbox.BaseCheckBox('Auto Assign Labels', self)
        self._extract_skin_faces_auto_assign_joints_labels_cbx = checkbox.BaseCheckBox('Auto Assign Labels', self)

        self._distance_widget = QWidget()
        distance_layout = layouts.VerticalLayout(spacing=2, margins=(5, 5, 5, 5))
        self._distance_widget.setLayout(distance_layout)
        self._distance_average_cbx = checkbox.BaseCheckBox('On Distance', self)
        self._average_falloff_curve = fallofcurve.FallofCurveWidget(parent=self)
        distance_layout.addWidget(self._distance_average_cbx)
        distance_layout.addWidget(self._average_falloff_curve)
        self._fast_delete_cbx = checkbox.BaseCheckBox('Fast Delete', self)

        self._average_falloff_widget.setVisible(False)
        self._mirror_auto_assign_joints_labels_cbx.setVisible(False)
        self._copy_skin_weights_auto_assign_joints_labels_cbx.setVisible(False)
        self._transfer_skin_uvs_auto_assign_joints_labels_cbx.setVisible(False)
        self._clean_skin_mesh_auto_assign_joints_labels_cbx.setVisible(False)
        self._extract_skin_faces_auto_assign_joints_labels_cbx.setVisible(False)
        self._distance_widget.setVisible(False)
        self._fast_delete_cbx.setVisible(False)

        self.main_layout.addWidget(self._average_falloff_widget)
        self.main_layout.addWidget(self._mirror_auto_assign_joints_labels_cbx)
        self.main_layout.addWidget(self._copy_skin_weights_auto_assign_joints_labels_cbx)
        self.main_layout.addWidget(self._transfer_skin_uvs_auto_assign_joints_labels_cbx)
        self.main_layout.addWidget(self._clean_skin_mesh_auto_assign_joints_labels_cbx)
        self.main_layout.addWidget(self._extract_skin_faces_auto_assign_joints_labels_cbx)
        self.main_layout.addWidget(self._distance_widget)
        self.main_layout.addWidget(self._fast_delete_cbx)

    def setup_signals(self):
        self._mirror_auto_assign_joints_labels_cbx.toggled.connect(self._controller.set_mirror_auto_assign_labels)
        self._copy_skin_weights_auto_assign_joints_labels_cbx.toggled.connect(
            self._controller.set_copy_skin_weights_auto_assign_labels)
        self._transfer_skin_uvs_auto_assign_joints_labels_cbx.toggled.connect(
            self._controller.set_transfer_skin_uvs_auto_assign_labels)
        self._clean_skin_mesh_auto_assign_joints_labels_cbx.toggled.connect(
            self._controller.set_clean_skin_mesh_auto_assign_labels)
        self._extract_skin_faces_auto_assign_joints_labels_cbx.toggled.connect(
            self._controller.set_extract_skin_faces_auto_assign_labels)
        self._distance_average_cbx.toggled.connect(self._controller.set_distance_average)
        self._fast_delete_cbx.toggled.connect(self._controller.set_fast_delete)
        self._average_falloff_curve.curveUpdated.connect(self._controller.set_average_weights_curve_points)

        self._model.mirrorAutoAssignLabelsChanged.connect(self._mirror_auto_assign_joints_labels_cbx.setChecked)
        self._model.copySkinWeightsAutoAssignLabelsChanged.connect(
            self._copy_skin_weights_auto_assign_joints_labels_cbx.setChecked)
        self._model.transferSkinUVsAutoAssignLabelsChanged.connect(
            self._transfer_skin_uvs_auto_assign_joints_labels_cbx.setChecked)
        self._model.cleanSkinMeshAutoAssignLabelsChanged.connect(
            self._clean_skin_mesh_auto_assign_joints_labels_cbx.setChecked)
        self._model.extractSkinFacesAutoAssignLabelsChanged.connect(
            self._extract_skin_faces_auto_assign_joints_labels_cbx.setChecked)
        self._model.useDistanceAverageChanged.connect(self._distance_average_cbx.setChecked)
        self._model.fastDeleteChanged.connect(self._fast_delete_cbx.setChecked)

    def _check_command_availability(self, command_name):

        client = self._controller.client
        if not client:
            return False

        if command_name == 'br_smooth_weights' or command_name == 'br_transfer_weights':
            for plugin_name in ['rampWeights.mll', 'weightsServer.mll', 'weightDriver.mll', 'brSmoothWeights.mll']:
                client.load_plugin(plugin_name, quiet=True)
            plugin_name = 'brSmoothWeights.mll'
            return client.is_plugin_loaded(plugin_name)
        elif command_name == 'ng_skin_tools':
            plugin_names = ['ngSkinTools2.mll', 'ngSkinTools.mll']
            for plugin_name in plugin_names:
                client.load_plugin(plugin_name, quiet=True)
                if client.is_plugin_loaded(plugin_name):
                    return True
            return False

        return True

    def _on_show_context_menu(self):
        super(SkinningWidget, self)._on_show_context_menu()

        self._average_falloff_curve.update_view()

    def refresh(self):

        self._mirror_auto_assign_joints_labels_cbx.setChecked(self._model.mirror_auto_assign_labels)
        self._copy_skin_weights_auto_assign_joints_labels_cbx.setChecked(
            self._model.copy_skin_weights_auto_assign_labels)
        self._transfer_skin_uvs_auto_assign_joints_labels_cbx.setChecked(
            self._model.transfer_uvs_auto_assign_labels)
        self._clean_skin_mesh_auto_assign_joints_labels_cbx.setChecked(
            self._model.clean_skin_mesh_auto_assign_labels)
        self._extract_skin_faces_auto_assign_joints_labels_cbx.setChecked(
            self._model.extract_skin_faces_auto_assign_labels)
        self._distance_average_cbx.setChecked(self._model.use_distance_average)
        self._fast_delete_cbx.setChecked(self._model.fast_delete)

        # NOTE: We do this to force model, to have point list value on startup
        self._controller.set_average_weights_curve_points(self._average_falloff_curve.curve_as_points())


class SkinningWidgetModel(QObject, object):

    mirrorAutoAssignLabelsChanged = Signal(bool)
    copySkinWeightsAutoAssignLabelsChanged = Signal(bool)
    transferSkinUVsAutoAssignLabelsChanged = Signal(bool)
    cleanSkinMeshAutoAssignLabelsChanged = Signal(bool)
    extractSkinFacesAutoAssignLabelsChanged = Signal(bool)
    useDistanceAverageChanged = Signal(bool)
    fastDeleteChanged = Signal(bool)

    def __init__(self):
        super(SkinningWidgetModel, self).__init__()

        self._mirror_auto_assign_labels = True
        self._copy_skin_weights_auto_assign_labels = True
        self._transfer_skin_uvs_auto_assign_labels = True
        self._clean_skin_mesh_auto_assign_labels = True
        self._extract_skin_faces_auto_assign_labels = True
        self._use_distance_average = True
        self._fast_delete = True
        self._average_weights_curve_points = list()

    @property
    def mirror_auto_assign_labels(self):
        return self._mirror_auto_assign_labels

    @mirror_auto_assign_labels.setter
    def mirror_auto_assign_labels(self, flag):
        self._mirror_auto_assign_labels = bool(flag)
        self.mirrorAutoAssignLabelsChanged.emit(self._mirror_auto_assign_labels)

    @property
    def copy_skin_weights_auto_assign_labels(self):
        return self._copy_skin_weights_auto_assign_labels

    @copy_skin_weights_auto_assign_labels.setter
    def copy_skin_weights_auto_assign_labels(self, flag):
        self._copy_skin_weights_auto_assign_labels = bool(flag)
        self.copySkinWeightsAutoAssignLabelsChanged.emit(self._copy_skin_weights_auto_assign_labels)

    @property
    def transfer_uvs_auto_assign_labels(self):
        return self._transfer_skin_uvs_auto_assign_labels

    @transfer_uvs_auto_assign_labels.setter
    def transfer_uvs_auto_assign_labels(self, flag):
        self._transfer_skin_uvs_auto_assign_labels = bool(flag)
        self.transferSkinUVsAutoAssignLabelsChanged.emit(self._transfer_skin_uvs_auto_assign_labels)()

    @property
    def clean_skin_mesh_auto_assign_labels(self):
        return self._clean_skin_mesh_auto_assign_labels

    @clean_skin_mesh_auto_assign_labels.setter
    def clean_skin_mesh_auto_assign_labels(self, flag):
        self._clean_skin_mesh_auto_assign_labels = bool(flag)
        self.cleanSkinMeshAutoAssignLabelsChanged.emit(self._clean_skin_mesh_auto_assign_labels)

    @property
    def extract_skin_faces_auto_assign_labels(self):
        return self._extract_skin_faces_auto_assign_labels

    @extract_skin_faces_auto_assign_labels.setter
    def extract_skin_faces_auto_assign_labels(self, flag):
        self._extract_skin_faces_auto_assign_labels = bool(flag)
        self.extractSkinFacesAutoAssignLabelsChanged.emit(self._extract_skin_faces_auto_assign_labels)

    @property
    def use_distance_average(self):
        return self._use_distance_average

    @use_distance_average.setter
    def use_distance_average(self, flag):
        self._use_distance_average = bool(flag)
        self.useDistanceAverageChanged.emit(self._use_distance_average)

    @property
    def fast_delete(self):
        return self._fast_delete

    @fast_delete.setter
    def fast_delete(self, flag):
        self._fast_delete = bool(flag)
        self.fastDeleteChanged.emit(self._fast_delete)

    @property
    def average_weights_curve_points(self):
        return self._average_weights_curve_points

    @average_weights_curve_points.setter
    def average_weights_curve_points(self, widget):
        self._average_weights_curve_points = widget


class SkinningWidgetController(object):

    def __init__(self, client, model):
        super(SkinningWidgetController, self).__init__()

        self._model = model
        self._client = client

    @property
    def model(self):
        return self._model

    @property
    def client(self):
        return self._client

    def set_mirror_auto_assign_labels(self, flag):
        self._model.mirror_auto_assign_labels = flag

    def set_copy_skin_weights_auto_assign_labels(self, flag):
        self._model.copy_skin_weights_auto_assign_labels = flag

    def set_transfer_skin_uvs_auto_assign_labels(self, flag):
        self._model.transfer_skin_uvs_auto_assign_labels = flag

    def set_clean_skin_mesh_auto_assign_labels(self, flag):
        self._model.clean_skin_mesh_auto_assign_labels = flag

    def set_extract_skin_faces_auto_assign_labels(self, flag):
        self._model.extract_skin_faces_auto_assign_labels = flag

    def set_distance_average(self, flag):
        self._model.use_distance_average = flag

    def set_fast_delete(self, flag):
        self._model.fast_delete = flag

    def set_average_weights_curve_points(self, points_list):
        self._model.average_weights_curve_points = points_list

    def smooth_bind_skin(self):
        return self._client.smooth_bind_skin(show_options=True)

    def rigid_bind_skin(self):
        return self._client.rigid_bind_skin(show_options=True)

    def detach_skin(self):
        return self._client.detach_bind_skin(show_options=False)

    def detach_skin_options(self):
        return self._client.detach_bind_skin(show_options=True)

    def open_paint_skin_weights_tool(self):
        return self._client.open_paint_skin_weights_tool(show_options=False)

    def open_paint_skin_weights_tool_options(self):
        return self._client.open_paint_skin_weights_tool(show_options=True)

    def mirror_skin_weights(self):
        auto_assign_labels = self._model.mirror_auto_assign_labels
        left_side, right_side = self._check_labels(auto_assign_labels)

        return self._client.mirror_skin_weights(
            auto_assign_labels=auto_assign_labels, left_side_label=left_side,
            right_side_label=right_side, show_options=True)

    def copy_skin_weights(self):
        auto_assign_labels = self._model.copy_skin_weights_auto_assign_labels
        left_side, right_side = self._check_labels(auto_assign_labels)

        return self._client.copy_skin_weights(
            auto_assign_labels=auto_assign_labels, left_side_label=left_side,
            right_side_label=right_side, show_options=True)

    def prune_skin_weights(self):
        return self._client.prune_skin_weights(show_options=True)

    def transfer_skin_uvs(self):
        auto_assign_labels = self._model.transfer_skin_uvs_auto_assign_labels
        left_side, right_side = self._check_labels(auto_assign_labels)

        return self._client.transfer_skin_uvs(
            auto_assign_labels=auto_assign_labels, left_side_label=left_side, right_side_label=right_side)

    def clean_skinned_mesh(self):
        auto_assign_labels = self._model.clean_skin_mesh_auto_assign_labels
        left_side, right_side = self._check_labels(auto_assign_labels)

        return self._client.clean_skinned_mesh(
            auto_assign_labels=auto_assign_labels, left_side_label=left_side, right_side_label=right_side)

    def combine_skinned_meshes(self):
        return self._client.combine_skinned_meshes()

    def extract_skinned_faces(self):
        auto_assign_labels = self._model.extract_skin_faces_auto_assign_labels
        left_side, right_side = self._check_labels(auto_assign_labels)

        return self._client.extract_skinned_selected_faces(
            auto_assign_labels=auto_assign_labels, left_side_label=left_side, right_side_label=right_side)

    def remove_unused_influences(self):
        return self._client.remove_unused_influences()

    def restore_bind_pose(self):
        res = qtutils.show_question(None, 'Go to Bind Pose', 'Are you sure yo want to restore bind pose?')
        if res != QMessageBox.Yes:
            return False

        return self._client.restore_bind_pose()

    def remove_bind_poses(self):
        res = qtutils.show_question(
            None, 'Remove Bind Poses', 'Are you sure yo want to remove all bind poses from the current scene?')
        if res != QMessageBox.Yes:
            return False

        return self._client.remove_bind_poses()

    def weights_hammer(self):
        return self._client.weights_hammer()

    def average_vertex_weights(self):
        use_distance_average = self._model.use_distance_average
        curve_weight_points = self._model.average_weights_curve_points

        return self._client.average_vertex_weights(
            use_distance=use_distance_average, curve_weight_points=curve_weight_points)

    def move_skin_weights(self):
        return self._client.move_skin_weights()

    def swap_skin_weights(self):
        return self._client.swap_skin_weights()

    def select_influences(self):
        return self._client.select_influences()

    def select_influence_components(self):
        return self._client.select_influence_components()

    def delete_influences(self):
        fast_delete = self._model.fast_delete
        return self._client.delete_influences(fast_delete=fast_delete)

    def br_smooth_weights(self):
        return self._client.br_smooth_weights()

    def br_smooth_weights_options(self):
        return self._client.br_smooth_weights(options=True)

    def br_transfer_weights_options(self):
        return self._client.br_transfer_weights()

    def br_transfer_weights(self):
        return self._client.br_transfer_weights(options=True)

    def ng_skin_tools(self):
        return self._client.ng_skin_tools()

    def _check_labels(self, auto_assign_labels):
        left_side = None
        right_side = None
        if auto_assign_labels:
            check_joints_labels = self._client.check_joints_labels()
            if not check_joints_labels:
                labels_dialog = labelsdialog.JointsLabelDialog()
                labels_dialog.exec_()
                left_side, right_side = labels_dialog.get_sides()
                if not left_side or not right_side:
                    return False, False

        return left_side, right_side
